@@ -2,21 +2,25 @@
 #'
 #' Compute the peaks-over-threshold censored negative log-likelihood function for the Brown--Resnick model.
 #'
-#' The function computes the censored log-likelihood function based on the representation
+#' The function computes the censored negative log-likelihood function based on the representation
 #' developed by Wadsworth et al. (2014) and Engelke et al. (2015). Margins must have been
-#' standardized, for instance to unit Frechet.
+#' standardized first, for instance to the unit Frechet scale.
 #'
 #' @author Raphael de Fondeville
 #' @param obs List of vectors for which at least one component exceeds a high threshold.
 #' @param loc Matrix of coordinates as given by \code{expand.grid()}.
 #' @param vario Semi-variogram function taking a vector of coordinates as input.
-#' @param u Vector of thresholds for censoring components.
+#' @param u Vector of threshold under which to censor components.
 #' @param p Number of samples used for quasi-Monte Carlo estimation. Must be a prime number.
-#' @param vec Generating vector for the quasi-Monte Carlo procedure. For a given \code{p} and dimensionality, 
+#' @param vec Generating vector for the quasi-Monte Carlo procedure. For a given prime \code{p} and dimension,
 #' can be computed using \code{genVecQMC}.
 #' @param nCores Number of cores used for the computation
 #' @param cl Cluster instance as created by \code{makeCluster} of the \code{parallel} package.
-#' @return Evaluation of the negative censored log-likelihood function for the set of observations \code{obs} and semi-variogram \code{vario}.
+#' @param likelihood vector of strings specifying the contribution. Either \code{"mgp"} for multivariate generalized Pareto,
+#'  \code{"poisson"} for a Poisson contribution for the observations falling below or \code{"binom"} for a binomial contribution.
+#' @param ntot integer number of observations below and above the threshold, to be used with Poisson or binomial likelihood
+#' @param ... Additional arguments passed to Cpp routine.
+#' @return Negative censored log-likelihood for the set of observations \code{obs} and semi-variogram \code{vario} with \code{attributes}  \code{exponentMeasure} for all of the \code{likelihood} type selected, in the order \code{"mgp"}, \code{"poisson"}, \code{"binom"}.
 #' @examples
 #' #Define semi-variogram function
 #' vario <- function(h){
@@ -44,44 +48,67 @@
 #'
 #'
 #' #Compute log-likelihood function
-#' censoredLikelihoodBR(exceedances, loc, vario, rep(thres, nrow(loc)), primeP, vec)
+#' censoredLikelihoodBR(obs = exceedances, loc = loc, vario = vario,
+#'  u = thres, p = primeP, vec = vec, ntot = 1000)
 #' @export
 #' @useDynLib mvPot mvtNormCpp
-#' @references Wadsworth, J. L. and J. A. Tawn (2014). Efficient Inference for Spatial Extreme Value Processes Associated to Log-Gaussian Random Function. Biometrika, 101(1):1-15.
-#' @references Asadi, P., Davison A. C. and S. Engelke (2015). Extremes on River Networks. Annals of Applied Statistics, 9(4), 2023-2050.
-
-censoredLikelihoodBR <- function(obs, 
-                              loc, 
-                              vario, 
-                              u, 
-                              p = 499L, 
-                              vec = NULL, 
-                              nCores = 1L, 
-                              cl = NULL){
+#' @references Wadsworth, J. L. and J. A. Tawn (2014). Efficient inference for spatial extreme value
+#'  processes associated to log-Gaussian random functions. Biometrika, 101(1), 1-15.
+#' @references Asadi, P., Davison A. C. and S. Engelke (2015). Extremes on River Networks.
+#'  Annals of Applied Statistics, 9(4), 2023-2050.
+censoredLikelihoodBR <- function(obs,
+                              loc,
+                              vario,
+                              u,
+                              p = 499L,
+                              vec = NULL,
+                              nCores = 1L,
+                              cl = NULL,
+                              likelihood = "mgp",
+                              ntot = NULL,
+                              ...){
+  likelihood <- match.arg(likelihood, choices = c("mgp", "poisson","binom"), several.ok = TRUE)
+  whichlik <- c("mgp", "poisson","binom") %in% likelihood
+  #Default for total number of observations is length of list
+  if(is.null(ntot) && is.list(obs)){
+    ntot <- length(obs)
+  }
+  if(is.matrix(obs)){ #Not converted to list
+    if(is.null(ntot)){
+      ntot <- nrow(obs)
+    }
+    #Keep only values above the threshold
+    obs <- obs[apply(obs, 1, function(vec){isTRUE(any(vec > u))}),]
+    obs <- split(obs, row(obs)) #create list
+  }
   if(is.null(vec) && isTRUE(all.equal(p, 499L))){
-    vec <- c(1, 209, 109, 191, 67, 51, 120, 93, 87, 157, 178, 45, 137, 84, 198, 
-              61, 232, 113, 182, 150, 57, 169, 141, 79, 132, 163, 38, 85, 131, 106, 
-              96, 165, 233, 179, 32, 228, 73, 233, 96, 131, 147, 32, 179, 165, 179, 
-              228, 32, 147, 165, 106, 228, 32, 179, 131, 32, 131, 228, 179, 106, 
-              165, 147, 179, 106, 147, 228, 165, 165, 179, 147, 228, 106, 106, 32, 
-              228, 147, 179, 32, 228, 106, 147, 32, 147, 228, 106, 179, 106, 179, 
-              228, 32, 147, 179, 32, 228, 106, 147, 147, 106, 179, 228, 32, 106, 
-              228, 147, 179, 32, 228, 147, 32, 179, 106, 32, 106, 179, 147, 147, 
-              179, 228, 106, 32, 106, 228, 179, 32, 147, 228, 179, 106, 32, 147, 
-              228, 106, 179, 32, 106, 147, 228, 179, 32, 228, 106, 179, 147, 32, 
-              147, 179, 106, 228, 106, 179, 147, 228, 32, 228, 179, 147, 106, 147, 
+    vec <- c(1, 209, 109, 191, 67, 51, 120, 93, 87, 157, 178, 45, 137, 84, 198,
+              61, 232, 113, 182, 150, 57, 169, 141, 79, 132, 163, 38, 85, 131, 106,
+              96, 165, 233, 179, 32, 228, 73, 233, 96, 131, 147, 32, 179, 165, 179,
+              228, 32, 147, 165, 106, 228, 32, 179, 131, 32, 131, 228, 179, 106,
+              165, 147, 179, 106, 147, 228, 165, 165, 179, 147, 228, 106, 106, 32,
+              228, 147, 179, 32, 228, 106, 147, 32, 147, 228, 106, 179, 106, 179,
+              228, 32, 147, 179, 32, 228, 106, 147, 147, 106, 179, 228, 32, 106,
+              228, 147, 179, 32, 228, 147, 32, 179, 106, 32, 106, 179, 147, 147,
+              179, 228, 106, 32, 106, 228, 179, 32, 147, 228, 179, 106, 32, 147,
+              228, 106, 179, 32, 106, 147, 228, 179, 32, 228, 106, 179, 147, 32,
+              147, 179, 106, 228, 106, 179, 147, 228, 32, 228, 179, 147, 106, 147,
               32, rep(179, 42)) / 499
     } else if(is.null(vec) && !isTRUE(all.equal(p, 499L))){
     stop("Invalid generating vector `vec`")
     }
-  if(class(obs) != "list" || length(obs) < 1 || class(obs[[1]]) != "numeric"){
+  if(!inherits(obs, "list") || length(obs) < 1 || !inherits(obs[[1]], c("numeric","integer"))){
     stop('obs must be a list of vectors')
   }
-  if(class(loc) != "data.frame") {
-    stop('loc must be the data frame of coordinates as generated by expand.grid()')
+  if(!inherits(loc, c("matrix", "data.frame"))) {
+    stop('`loc` must be a data frame of coordinates as generated by `expand.grid()` or a matrix of locations (one site per row)')
   }
   if(length(u) == 1L){
     u <- rep(u, nrow(loc))
+  } else{
+   if(abs(max(u) - min(u)) >  1e-10){
+     warning("The threshold level on the unit Frechet scale in vector `u` should be the same for all components!")
+  }
   }
   n <- length(obs)
   D <- nrow(loc)
@@ -90,7 +117,7 @@ censoredLikelihoodBR <- function(obs,
     stop('The size of the vectors of observations does not match grid size.')
   }
   if(!is.numeric(u)  || length(u) != D) {
-    stop('`u` must be a vector with a length equal to the number of location.')
+    stop('`u` must be a vector whose length is equal to the number of location.')
   }
   if(!is.numeric(p)) {
     stop('`p` must be a numeric.')
@@ -101,10 +128,16 @@ censoredLikelihoodBR <- function(obs,
   if(!is.numeric(nCores) || nCores < 1) {
     stop('`nCores` must a positive number of cores to use for parallel computing.')
   }
-  if(nCores > 1 && length(grep("cluster", class(cl))) == 0) {
+  if(nCores > 1 && !inherits(cl, "cluster")) {
     stop('For parallel computation, `cl` must an cluster created by `makeCluster` of the package parallel.')
   }
-
+  ellipsis <- list(...)
+  if(!is.null(ellipsis$nrep)){
+    nrep <- as.integer(ellipsis$nrep)
+    #number of Monte-Carlo replications over which to average calculations to estimate error. Default to 10.
+  } else{ 
+    nrep <- 10L
+  }
   gamma <- tryCatch({
     dists <- lapply(1:ncol(loc), function(i) {
       outer(loc[, i], loc[, i], "-")
@@ -134,13 +167,15 @@ censoredLikelihoodBR <- function(obs,
       upperBound = sqrt(gamma[-i, i]/2) # - log(thres[i]/thres[-i])/sqrt(2*gamma[-i, i]) ##superfluous b/c log(1)=0
       cov = (gamma[-i, i] %*% t(identityVector) + t(gamma[i, -i] %*% t(identityVector)) - gamma[-i, -i]) / (2*sqrt(gamma[-i, i] %*% t(identityVector)*t(gamma[i, -i] %*% t(identityVector))))
 
-      tmp <-.C(mvtNormCpp, 
-               as.integer(p), 
-               as.integer(length(upperBound)), 
-               as.double(cov), 
-               as.double(upperBound), 
-               as.double(vec[1:length(upperBound)]), 
-               est = double(length=1), 
+      tmp <-.C(mvtNormCpp,
+               as.integer(p),
+               as.integer(length(upperBound)),
+               as.double(cov),
+               as.double(upperBound),
+               as.double(vec[1:length(upperBound)]),
+               as.integer(nrep),
+               as.integer(FALSE),
+               est = double(length=1),
                err = double(length=1),
                PACKAGE = "mvPot"
       )
@@ -171,10 +206,10 @@ censoredLikelihoodBR <- function(obs,
         logdetA = determinant(sigma[posAboveShifted, posAboveShifted, drop = FALSE], logarithm = TRUE)$modulus
         omega <- log(observation[posAbove][-1]/observation[posAbove][1]) + gamma[posAbove[-1], posAbove[1]]
 
-        mle1 <- 1 / 2 * (logdetA + log((2 * pi)^(k-1)) + t(omega)  %*%  invCovMat  %*%  omega)  + log(obs[[j]][posAbove][1]) + sum(log(obs[[j]][posAbove]))
+        mle1 <- 1 / 2 * (logdetA + (k-1) * log(2 * pi) + t(omega)  %*%  invCovMat  %*%  omega)  + log(observation[posAbove][1]) + sum(log(observation[posAbove]))
       } else {
         #One exceedance only -> parameters have no impact
-        mle1 <- 2 * log(obs[[j]][posAbove][1]) #not divided by u?
+        mle1 <- 2 * log(observation[posAbove][1]) 
       }
 
       if(k < D){
@@ -190,13 +225,15 @@ censoredLikelihoodBR <- function(obs,
           mle2 = max(1e-323, tmp)
           mle2 = - log(mle2)
         } else {
-          tmp <- .C(mvtNormCpp, 
-                    as.integer(p), 
-                    as.integer(length(muC)), 
-                    as.double(sigmaC), 
-                    as.double(as.vector(muC)), 
-                    as.double(vec[1:length(muC)]), 
-                    est = double(length=1), 
+          tmp <- .C(mvtNormCpp,
+                    as.integer(p),
+                    as.integer(length(muC)),
+                    as.double(sigmaC),
+                    as.double(as.vector(muC)),
+                    as.double(vec[1:length(muC)]),
+                    as.integer(nrep),
+                    as.integer(FALSE),
+                    est = double(length=1),
                     err = double(length=1),
                     PACKAGE = "mvPot"
           )
@@ -224,12 +261,23 @@ censoredLikelihoodBR <- function(obs,
     }
 
     blockSize <- floor((D + n) / nCores) + 1
-    prod <- parallel::parLapply(cl, 1:(nCores), blockedMLE)
+    pro <- parallel::parLapply(cl, 1:(nCores), blockedMLE)
   } else {
-    prod <- lapply(1:(D + n), mleEst)
+    pro <- lapply(1:(D + n), mleEst)
   }
-
-  res <- (n) * log(sum(unlist(prod)[1:D])) + sum(unlist(prod)[(D + 1):(n + D)])
+  exponentMeasure <- sum(unlist(pro)[1:D] / u)
+  if(any(whichlik[2:3]) && ntot == n){
+    warning("Total number of observations currently same as number of exceedances.")
+  }
+  if(whichlik[3] && exponentMeasure > 1){
+    warning("Exponent measure is greater than 1.")
+  }
+  res <- sum(unlist(pro)[(D + 1):(n + D)]) + 
+    suppressWarnings(c(n * log(exponentMeasure), 
+                       -ntot * exponentMeasure, 
+                       (ntot - n) * log(1 - exponentMeasure)
+                       )[whichlik])
+  attributes(res) <- list("ExponentMeasure" = exponentMeasure, names = c("mgp", "poisson", "binom")[whichlik])
   return(res)
 }
 
@@ -237,8 +285,8 @@ censoredLikelihoodBR <- function(obs,
 #' @export
 #' @rdname censoredLikelihoodBR
 censoredLikelihood <- function(obs, loc, vario, u, p = 499L, vec = NULL, nCores = 1L, cl = NULL){
-  .Deprecated(new = "censoredLikelihood", package = "mvPot", 
-              msg = "Please use the function `censoredLikelihoodBR` to estimate the censored likelihood of the Brown-Resnick processes.", 
+  .Deprecated(new = "censoredLikelihood", package = "mvPot",
+              msg = "Please use the function `censoredLikelihoodBR` to estimate the censored likelihood of the Brown-Resnick process.",
               old = "censoredLikelihood")
   censoredLikelihoodBR(obs = obs, loc = loc, vario = vario, u = u, p = p, vec = vec, nCores= nCores, cl = cl)
-}
\ No newline at end of file
+}
